import { forwardRef, EventEmitter, Component, ViewEncapsulation, IterableDiffers, Renderer2, Input, Output, ViewChild, NgModule } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import cloneDeep from 'lodash.clonedeep';

const SELECTIZE_VALUE_ACCESSOR = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef(() => NgSelectizeComponent),
    multi: true
};
class NgSelectizeComponent {
    constructor(_differs, renderer) {
        this._differs = _differs;
        this.renderer = renderer;
        this.enabled = true;
        this.onBlur = new EventEmitter(false);
    }
    ngOnInit() { }
    ngAfterViewInit() {
        if (this.id && this.id.length > 0) {
            this.renderer.setAttribute(this.selectizeInput.nativeElement, 'id', this.id);
        }
        this.reset();
    }
    reset() {
        this.selectize = $(this.selectizeInput.nativeElement).selectize(this.config)[0].selectize;
        this.selectize.on('change', this.onSelectizeValueChange.bind(this));
        this.selectize.on('blur', this.onBlurEvent.bind(this));
        this.updatePlaceholder();
        this.onEnabledStatusChange();
    }
    /**
     * Change detection for primitive types.
     */
    ngOnChanges(changes) {
        if (this.selectize) {
            if (changes.hasOwnProperty('placeholder') || changes.hasOwnProperty('hasOptionsPlaceholder')
                || changes.hasOwnProperty('noOptionsPlaceholder')) {
                this.updatePlaceholder();
            }
            if (changes.hasOwnProperty('enabled')) {
                this.onEnabledStatusChange();
            }
        }
    }
    /**
     * Implementing deep check for option comparison
     *
     * FIXME -> Implement deep check to only compare against label and value fields.
     */
    ngDoCheck() {
        if (this.selectize) {
            if (this._options_differ) {
                const changes = this._options_differ.diff(this._options);
                if (changes) {
                    this._applyOptionsChanges(changes);
                }
            }
            if (this._optgroups_differ) {
                const changes = this._optgroups_differ.diff(this._optgroups);
                if (changes) {
                    this._applyOptionGroupChanges(changes);
                }
            }
        }
    }
    _applyOptionsChanges(changes) {
        changes.forEachAddedItem((record) => {
            this.onSelectizeOptionAdd(record.item);
        });
        changes.forEachRemovedItem((record) => {
            this.onSelectizeOptionRemove(record.item);
        });
        this.updatePlaceholder();
        this.evalHasError();
    }
    _applyOptionGroupChanges(changes) {
        changes.forEachAddedItem((record) => {
            this.onSelectizeOptGroupAdd(record.item);
        });
        changes.forEachRemovedItem((record) => {
            this.onSelectizeOptGroupRemove(record.item);
        });
        this.updatePlaceholder();
        this.evalHasError();
    }
    onBlurEvent() {
        if (this.formControl) {
            this.formControl.markAsTouched();
        }
        this.onBlur.emit();
        this.evalHasError();
    }
    onSelectizeOptGroupAdd(optgroup) {
        this.selectize.addOptionGroup(optgroup[this.getOptgroupField()], optgroup);
    }
    onSelectizeOptGroupRemove(optgroup) {
        this.selectize.removeOptionGroup(optgroup[this.getOptgroupField()]);
    }
    /**
     * Refresh selected values when options change.
     */
    onSelectizeOptionAdd(option) {
        var _a;
        (_a = this.selectize) === null || _a === void 0 ? void 0 : _a.addOption(cloneDeep(option));
        const valueField = this.getValueField();
        if (this.value) {
            const items = (typeof this.value === 'string' || typeof this.value === 'number') ? [this.value] : this.value;
            if (items && items instanceof Array && items.find(value => value === option[valueField])) {
                this.selectize.addItem(option[valueField], true);
            }
        }
    }
    onSelectizeOptionRemove(option) {
        this.selectize.removeOption(option[this.getValueField()]);
    }
    evalHasError() {
        const parent = $(this.selectize.$control).parent();
        if (this.formControl) {
            if (this.formControl.touched && this.formControl.invalid) {
                parent.addClass(this.errorClass || 'has-error');
            }
            else if (parent.hasClass('has-error')) {
                parent.removeClass(this.errorClass || 'has-error');
            }
        }
    }
    /**
     * Update the current placeholder based on the given input parameter.
     */
    updatePlaceholder() {
        if (this.selectize.items.length === 0 && this.selectize.settings.placeholder !== this.getPlaceholder()) {
            this.selectize.settings.placeholder = this.getPlaceholder();
            this.selectize.updatePlaceholder();
            this.selectize.showInput(); // Without this, when options are cleared placeholder only appears after focus.
        }
    }
    /**
     * Called when a change is detected in the 'enabled' input field.
     * Sets the selectize state based on the new value.
     */
    onEnabledStatusChange() {
        this.enabled ? this.selectize.enable() : this.selectize.disable();
    }
    /**
     * Dispatches change event when a value change is detected.
     * @param $event
     */
    onSelectizeValueChange($event) {
        // In some cases this gets called before registerOnChange.
        if (this.onChangeCallback) {
            this.onChangeCallback(this.selectize.getValue());
        }
    }
    /**
     * Returns the applicable placeholder.
     */
    getPlaceholder() {
        if (this.hasOptionsPlaceholder) {
            if (this.options && this.options.length > 0) {
                return this.hasOptionsPlaceholder;
            }
        }
        if (this.noOptionsPlaceholder) {
            if (!this.options || this.options.length === 0) {
                return this.noOptionsPlaceholder;
            }
        }
        return this.placeholder;
    }
    /**
     * Implementation from ControlValueAccessor
     *
     * Empty check on 'obj' removed due to restriction on resetting the field.
     * From testing, async should still function appropriately.
     *
     * FIXME This might not be necessary anymore..
     *
     * @param obj
     */
    writeValue(obj) {
        var _a;
        if (obj !== this.value) {
            this.value = obj;
        }
        (_a = this.selectize) === null || _a === void 0 ? void 0 : _a.setValue(this.value);
    }
    /**
     * Implementation from ControlValueAccessor, callback for (ngModelChange)
     * @param fn
     */
    registerOnChange(fn) {
        this.onChangeCallback = fn;
    }
    /**
     * Implementation from ControlValueAccessor
     * @param fn
     */
    registerOnTouched(fn) {
        this.onTouchedCallback = fn;
    }
    getValueField() {
        return this.config['valueField'] ? this.config['valueField'] : 'value';
    }
    getOptgroupField() {
        return this.config['optgroupField'] ? this.config['optgroupField'] : 'optgroup';
    }
    set options(value) {
        this._options = value;
        if (!this._options_differ && value) {
            this._options_differ = this._differs.find(value).create();
        }
    }
    get options() {
        return this._options;
    }
    set optgroups(value) {
        this._optgroups = value;
        if (!this._optgroups_differ && value) {
            this._optgroups_differ = this._differs.find(value).create();
        }
    }
    get optgroups() {
        return this._optgroups;
    }
}
NgSelectizeComponent.decorators = [
    { type: Component, args: [{
                selector: 'ng-selectize',
                template: `<select #selectizeInput></select>`,
                providers: [SELECTIZE_VALUE_ACCESSOR],
                encapsulation: ViewEncapsulation.None
            },] }
];
NgSelectizeComponent.ctorParameters = () => [
    { type: IterableDiffers },
    { type: Renderer2 }
];
NgSelectizeComponent.propDecorators = {
    config: [{ type: Input }],
    id: [{ type: Input }],
    placeholder: [{ type: Input }],
    hasOptionsPlaceholder: [{ type: Input }],
    noOptionsPlaceholder: [{ type: Input }],
    enabled: [{ type: Input }],
    value: [{ type: Input }],
    formControl: [{ type: Input }],
    errorClass: [{ type: Input }],
    onBlur: [{ type: Output }],
    selectizeInput: [{ type: ViewChild, args: ['selectizeInput',] }],
    options: [{ type: Input }],
    optgroups: [{ type: Input }]
};

class NgSelectizeModule {
}
NgSelectizeModule.decorators = [
    { type: NgModule, args: [{
                declarations: [NgSelectizeComponent],
                exports: [NgSelectizeComponent]
            },] }
];

/*
 * Public API Surface of ng-selectize
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NgSelectizeComponent, NgSelectizeModule, SELECTIZE_VALUE_ACCESSOR };
//# sourceMappingURL=ng-selectize.js.map
